#!/bin/python

import os
import shvar

# Directories that we want to empty but not delete
directories_to_be_emptied = [
    '/root',
    '/proc',
    '/lost+found',
    '/var/gdm',
    ]

# Directories that we want to empty of files
directories_to_be_cleaned = [
    '/var/log', ## FIXME: Think this through
    ]

# files that need removing
files_to_be_removed = [
    '/.autofsck',
    ]

def clear_rpm_db_files (rootdir):
    """Removes rpm database files in the destination if they exist"""
    def rmpath (p):
        try: os.remove (p)
        except OSError: pass
    rmpath (os.path.join (rootdir, 'var/lib/rpm/__db.001'))
    rmpath (os.path.join (rootdir, 'var/lib/rpm/__db.002'))
    rmpath (os.path.join (rootdir, 'var/lib/rpm/__db.003'))

def clean_directory (dir, remove_when_done=True, keep_child_directories=False):
    """Removes all files in a directory, optionally removing the directory afterwards"""
    assert os.path.isdir and not os.path.islink (dir)
    file_list = os.listdir (dir)

    for f in file_list:
        path = os.path.join (dir, f)
        if os.path.isdir (path) and not os.path.islink (path):
            clean_directory (path,
                             not keep_child_directories,
                             keep_child_directories)
        else:
            os.remove (path)
    
    if remove_when_done:
        os.rmdir (dir)

def clean_up_directories (rootdir, to_empty, to_clean, to_remove):
    for d in to_empty:
        path = os.path.normpath (rootdir + '/' + d)
        if not os.path.isdir (path):
            continue
        clean_directory (path, False)

    for d in to_clean:
        path = os.path.normpath (rootdir + '/' + d)
        if not os.path.isdir (path):
            continue
        clean_directory (path, False, True)

    for f in to_remove:
        path = os.path.join (rootdir, f)
        if not os.path.isfile (path):
            continue
        os.remove (path)

def clean_up_root (rootdir):
    # FIXME: Do this nicer at some point
    os.system ("chroot %s /usr/bin/rsync -a /etc/skel/ /root/" % rootdir)
    os.chmod (rootdir + '/root', 0750)

def sysconfig_file (rootdir, name):
    path = os.path.normpath (rootdir + '/etc/sysconfig/' + name)
    return shvar.shvarFile (path)

def tweak_readonly_root_config (rootdir):
    shf = sysconfig_file (rootdir, "readonly-root")
    # run rc.readonly
    shf["READONLY"] = "yes"
    shf.write ()
    
def tweak_udev_config (rootdir):
    shf = sysconfig_file (rootdir, "udev")
    # Mount a ramfs over /dev
    shf["UDEV_RAMFS"] = "yes"
    # We aren't using an initrd
    shf["UDEV_INITRD"] =  "no"
    # Don't keep the ramfs from the initrd
    shf["UDEV_KEEP_DEV"] = "no"
    shf.write ()
        
def tweak_firstboot_config (rootdir):
    shf = sysconfig_file (rootdir, "firstboot")
    # Never run firstboot
    shf["RUN_FIRSTBOOT"] = "no"
    shf.write ()

def tweak_kudzu_config (rootdir):
    shf = sysconfig_file (rootdir, "kudzu")
    # Take default actions without prompting
    shf["KUDZU_ARGS"] =  "-q"
    shf.write ()

def prepare_diskless_root (rootdir):
    clear_rpm_db_files (rootdir)
    clean_up_directories (rootdir,
                          directories_to_be_emptied,
                          directories_to_be_cleaned,
                          files_to_be_removed)
    clean_up_root (rootdir)
    tweak_readonly_root_config (rootdir)
    tweak_udev_config (rootdir)
    tweak_firstboot_config (rootdir)
    tweak_kudzu_config (rootdir)

if __name__ == '__main__':
    import sys
    import getopt

    def Usage ():
        print "Usage: readonly-prepare <diskless-rootdir>"

    if os.getuid () != 0:
        print "Please restart %s with root permissions!" % (sys.argv[0])
        sys.exit (1)

    if len (sys.argv) != 2:
        Usage ()
        sys.exit (1)

    prepare_diskless_root (sys.argv[1])
